%=================================
% CODE sdq
%
% steady one-dimensional diffusion
% with quadratic elements
%=================================

%-----------
% input data
%-----------

L=1.0; k=1.0; q0 = -1.0; fL=0.0;

helm=0;  % Helmholtz constant: set to 0 for the Poisson equation

q0=-pi; fL=0.0; helm=pi^2;   % CASE B

ne=6; ratio=1.0;

%----------------
% grid generation
%----------------

xe = elm_line1 (0,L,ne,ratio);

%------------------------------
% number of unique global nodes
%------------------------------

ng = 2*ne+1;

%--------------------------
% generate the global nodes
%--------------------------

Ic = 0;  % counter

for i=1:ne
 Ic = Ic+1; xg(Ic) = xe(i);
 Ic = Ic+1; xg(Ic) = 0.5*(xe(i)+xe(i+1));
end

xg(ng) = xe(ne+1);

%-------------------
% specify the source
%-------------------

for i=1:ng
 s(i) = 10.0*exp(-5.0*xg(i)^2/L^2);
 s(i) = 0.0;   % CASE B
end

%-------------------------
% compact element assembly
%-------------------------

[ap,bp,cp,dp,ep,b] = sdq_sys (ne,xe,q0,fL,k,s, helm);

%--------------
% linear solver
%--------------

f = penta (ng-1,ap,bp,cp,dp,ep,b);

f(ng) = fL;

%-----
% plot
%-----

plot(xg, f,'-o');
hold on;
plot(xe, zeros(ne+1,1),'x');
xlabel('x'); ylabel('f');

%-----
% done
%-----
