%=================================
% CODE sdqm
%
% steady one-dimensional diffusion
% with the quadratic modal expansion
%=================================

%-----------
% input data
%-----------

L=1.0;
k=1.0; q0 = -1.0; fL=0.0;

ne=16; ratio=5.0;
ne=32; ratio=2.0;
ne=4; ratio=5.0;
ne=1; ratio=5.0;

%----------------
% grid generation
%----------------

xe = elm_line1 (0,L,ne,ratio);

%------------------------
% number of global modes
%-----------------------

ng = 2*ne+1;

%--------------------------
% generate the global nodes
%--------------------------

Ic = 0;  % counter

for i=1:ne
  Ic = Ic+1; xg(Ic) = xe(i);
  Ic = Ic+1; xg(Ic) = 0.5*(xe(i)+xe(i+1));
end

xg(ng) = xe(ne+1);

%-------------------
% specify the source
%-------------------

for i=1:ng
 s(i) = 10.0*exp(-5.0*xg(i)^2/L^2);
end

%---------------------------------------------
% compute the modal coefficients of the source
% by collocation at the element mid-point
%---------------------------------------------

for i=1:ng
 sc(i) = s(i);
end

for i=1:ne
 j=2*i;
 sc(j) = (-s(j-1)+2.0*s(j)-s(j+1))/2.0;
end

%-----------------------------------------------------
% compact element assembly of the pentadiagonal system
%-----------------------------------------------------

[ap,bp,cp,dp,ep,rp] = sdqm_sys (ne,xe,q0,fL,k,sc);

%------
% formulate and solve the tridiaginal system
%------

for i=1:ne
 j=2*i-1;
 at(i)=ap(j); bt(i)=cp(j); ct(i)=ep(j); rt(i)=rp(j);
end

sol = thomas (ne,at,bt,ct,rt);

for i=1:ne
 j=2*i-1;
 f(j)=sol(i);
end

f(ng) = fL;

%------
% solve for the even-numbered unknowns
% and recover the midpoint values
%------

for i=1:ne
 j=2*i;
 f(j)=rp(j)/ap(j);
 f(j) = (f(j-1)+2.0*f(j)+f(j+1))/2.0;
end

%-----
% plot
%-----

plot(xg, f,'-o');
hold on;
plot(xe, zeros(ne+1,1),'x');
xlabel('x');ylabel('f');

%-----
% done
%-----
