function [xe,xen,xg,c,ng] = discr_s(x1,x2,ne,ratio,np)

%========================================
% Discretize the interval (x1, x2)
% into "ne" elements,
% and generate the np-order
% Lobatto element interpolation nodes
%
% xe:  element end-nodes
% xen: element interpolation nodes
% xg:  unique global nodes
% c:   connectivity matrix
% ng:  number of unique global nodes
%========================================

%-----------------------------
% Define the element end-nodes
%-----------------------------

xe = elm_line1(x1,x2,ne,ratio);

%---------------------------------------------
% Define the element interpolation nodes (xen)
%---------------------------------------------

for l=1:ne

  mx = 0.5*(xe(l+1)+xe(l));
  dx = 0.5*(xe(l+1)-xe(l));

  xen(l,1) = xe(l);

  if(np(l) >1)

    [zlob, wlob] = lobatto(np(l)-1);

    for je=1:np(l)-1
      xen(l,je+1) = mx + zlob(je)*dx;
    end
  end

  xen(l,np(l)+1) = xe(l+1);

end

%-------------------------------
% define the connectivity matrix
% and the global nodes
%-------------------------------

Ic = 2;      % global node counter

for l=1:ne
  Ic = Ic-1;
  for je=1:np(l)+1
    c(l,je) = Ic;
    xg(Ic) = xen(l,je);
    Ic = Ic+1;
  end
end

%----------------------
% total number of nodes
%----------------------

ng = Ic-1;

%-----
% done
%-----

return;
