%==============================================
% CODE: sde
%
% Steady 1D diffusion with evenly spaced nodes
% over the elements,
% deriving from the modal expansion
%==============================================

%-----------
% input data
%-----------

L=1.0; k=1.0; fL=0; q0=-1.0;
helm=0;  % Helmholtz constant: set to 0 for Poisson

%---
% case studies
%
fL = exp(1.0); q0=-1;   % CASE A
fL=1/26.0; q0=-100/26^2; % CASE R and R1
fL=0.0; q0=-pi;  helm=pi^2;  % CASE H
%---

ne=2; ratio=1.0;               % number of elements, stretch ratio

for i=1:ne
 np(i)=8;   % default element polynomial order
end
%np(1)=4; np(2)=4; np(3)=4;  % element polynomial order

%-----
% trap
%-----

for i=1:ne

   if(np(i)>8)
   disp(' -->');
   disp(' max polynomial order is 8');
   error(' sdsm: Sorry this high order not yet implemented');
   end

end

%-----------
% discretize
%-----------

[xe,xen,xg,c,ng] = discr_e (0,L,ne,ratio,np);

%-------------------
% specify the source
%-------------------

for i=1:ng
  s(i) = 10.0*exp(-5.0*xg(i)^2/L^2);
  s(i) = -exp(xg(i));              % CASE A
  s(i) = 200*(1-75*(2*xg(i)-1)^2)/(1+25*(2*xg(i)-1)^2)^3 ;   % case R1
  s(i) = 1.0/(1+25*(2*xg(i)-1)^2);   % CASE R
  s(i)=0.0;          % CASE H
end

%-----------------
% element assembly
%-----------------

[gdm,b] = sde_sys (ne,xe,np,ng,c,q0,fL,k,s,helm);

[xe,xen,xg,c,ng] = discr_e (0,L,ne,ratio,np);

%--------------
% linear solver
%--------------

gdm(:,ng) = [];  % remove the last (ng) column
gdm(ng,:) = [];  % remove the last (ng) row
b(:,ng) = [];    % remove the last (ng) element

f = b/gdm';       % solve the linear system

f = [f fL];      % add the value at the right end

%-----
% plot
%-----

plot(xg, f,'-+');hold on;

% ye= zeros(ne+1,1); plot(xe,ye,'*');

xlabel('x'); ylabel('f');

%-----
% done
%-----
