%=======================================
% CODE: sdms
%
% Steady 1D diffusion with the
% modal expansion and spectral elements
%=======================================

%-----------
% input data
%-----------

L=1.0; k=1.0; fL=0; q0=-1.0;

helm=0;   % Helmholtz constant: set to 0 for the Poisson equation

%---
% case studies
%
fL=1/26.0; q0=-100/26^2;    % CASE R and R1
fL=0; q0=-pi; helm=pi^2;    % CASE Helmholtz
fL=exp(1.0); q0=-1.0;       % CASE B
%---

ne=2; ratio=1.0;               % number of elements, stretch ratio

for i=1:ne
 np(i)=4;   % default element polynomial order
end

%-----
% trap
%-----

for i=1:ne

   if(np(i)>7)
   disp(' -->');
   disp(' max polynomial order is 7');
   error(' sdms: Sorry this high order not yet implemented');
   end

end

%------------------------
% element node generation 
%------------------------

[xe,xen,xg,c,ng] = discr_s (0,L,ne,ratio,np);

%-------------------
% specify the source
%-------------------

for i=1:ng
  s(i) = 10.0*exp(-5.0*xg(i)^2/L^2);
  s(i) = -exp(xg(i));                    % CASE B
  s(i) = 200*(1-75*(2*xg(i)-1)^2)/(1+25*(2*xg(i)-1)^2)^3 ;   % case R1
  s(i) = 0;         % CASE H
  s(i) = 1.0/(1+25*(2*xg(i)-1)^2);       % CASE R
end

%------------------------------------------------
% compute the coefficients of the modal expansion
% for the source by solving a linear system
%
% use the Vandermonde matrix for higher orders
%------------------------------------------------

Jc = 0;

for l=1:ne

   Jc=Jc+1;    % first end-node
   sc(Jc)=s(Jc);

%---
  if(np(l)==2)  % quadratic element: one interior mode
%---

   Jc=Jc+1; 
   sc(Jc) = (-s(Jc-1)+2.0*s(Jc)-s(Jc+1))/2.0;

%---
  elseif(np(l)==3)  % cubic element: 2 interior modes
%---

   Istart = Jc;
   Iend   = Jc+np(l);

   xi=-1/sqrt(5);
   vd(1,1)=(1-xi^2);
   vd(2,1)=3*(1-xi^2)*xi;
   rhs(1)=s(Jc+1)-s(Istart)*(1-xi)/2-s(Iend)*(1+xi)/2;

   xi= 1/sqrt(5);
   vd(1,2)=(1-xi^2);
   vd(2,2)=3*(1-xi^2)*xi;
   rhs(2)=s(Jc+2)-s(Istart)*(1-xi)/2-s(Iend)*(1+xi)/2;

   local = rhs/vd;

   Jc=Jc+1; 
   sc(Jc)=local(1);
   Jc=Jc+1; 
   sc(Jc)=local(2);

%---
  elseif(np(l)==4)  % quartic element: 3 interior modes
%---

   Istart = Jc;
   Iend   = Jc+np(l);

   xi=-sqrt(3/7);
   vd(1,1)=(1-xi^2);
   vd(2,1)=3*(1-xi^2)*xi;
   vd(3,1)=3*(1-xi^2)*(5*xi^2-1)/2;
   rhs(1)=s(Jc+1)-s(Istart)*(1-xi)/2-s(Iend)*(1+xi)/2;

   xi= 0;
   vd(1,2)=(1-xi^2);
   vd(2,2)=3*(1-xi^2)*xi;
   vd(3,2)=3*(1-xi^2)*(5*xi^2-1)/2;
   rhs(2)=s(Jc+2)-s(Istart)*(1-xi)/2-s(Iend)*(1+xi)/2;

   xi= sqrt(3/7);
   vd(1,3)=(1-xi^2);
   vd(2,3)=3*(1-xi^2)*xi;
   vd(3,3)=3*(1-xi^2)*(5*xi^2-1)/2;
   rhs(3)=s(Jc+3)-s(Istart)*(1-xi)/2-s(Iend)*(1+xi)/2;

   local = rhs/vd;

   Jc=Jc+1;
   sc(Jc)=local(1);
   Jc=Jc+1;
   sc(Jc)=local(2);
   Jc=Jc+1;
   sc(Jc)=local(3);

  else

   disp(' -->');
   disp(' max polynomial order is 4');
   error(' sdms: Sorry this high order not yet implemented');

  end

   sc(ng)=s(ng);  % last global node

end

%-----------------
% element assembly
%-----------------

[gdm,b] = sdms_sys (ne,xe,np,ng,c,q0,fL,k,sc,helm);

%--------------
% linear solver
%--------------

gdm(:,ng) = [];  % remove the last (ng) column
gdm(ng,:) = [];  % remove the last (ng) row
b(:,ng) = [];    % remove the last (ng) element

f = b/gdm';       % solve the linear system

f = [f fL];      % add the value at the right end

%-----------------------
% plot the end-node data
%-----------------------

Ic=1;
for l=1:ne
 plot(xg(Ic), f(Ic),'o');hold on;
 Ic=Ic+np(l);
end
plot(xg(Ic), f(Ic),'x');hold on;

xlabel('x'); ylabel('f');
ye= zeros(ne+1,1);
plot(xe,ye,'o');

%-----
% done
%-----
