function [gdm,b] = sdms_sys (ne,xe,np,ng,c,q0,fL,k,s,helm)

%--------------------------------------
% Assembly of the linear system
% for one-dimensional Steady Diffusion
% using the modal expansion (sdsm)
% and spectral elements
%
% gdm: global diffusion matrix
% b:   right-hand side
% c:   connectivity matrix
%--------------------------------------

%-------------
% element size
%-------------

for l=1:ne
  h(l) = xe(l+1)-xe(l);
end

%-----------
% initialize
%-----------

gdm = zeros(ng,ng);
b = zeros(1,ng);    % RHS

b(1) = q0/k;    % Neumann condition on the left

%-----------------------
% loop over the elements
%-----------------------

for l=1:ne

   m = np(l);

% compute the mass matrix

  clear elm_mm

   for i=1:m+1
     for j=1:m+1
       elm_mm(i,j)= 0;
     end
   end

   for i=2:m
    elm_mm(i,i) = h(l)* 2 *(i-1)^2*i^2/((2*i-3)*(2*i-1)*(2*i+1));
    elm_mm(i,i+2) = - h(l)* (i-1)*i*(i+1)*(i+2)/((2*i-1)*(2*i+1)*(2*i+3));
   end

   for i=3:m
    elm_mm(i,i-2) = - h(l)* (i-3)*(i-2)*(i-1)*i/((2*i-5)*(2*i-3)*(2*i-1));
   end

   elm_mm(1,1) = h(l)/3;
   elm_mm(1,2) = h(l)/3;
   elm_mm(2,1) = h(l)/3;
   elm_mm(1,3) = - h(l)/5;
   elm_mm(3,1) = - h(l)/5;

   elm_mm(1,m+1) = h(l)/6;
   elm_mm(2,m+1) = h(l)/3;
   elm_mm(3,m+1) = h(l)/5;
   elm_mm(m+1,1) = h(l)/6;
   elm_mm(m+1,2) = h(l)/3;
   elm_mm(m+1,3) = h(l)/5;
   elm_mm(m+1,m+1) = h(l)/3;

%   elm_mm  % view

% compute the diffusion matrix

  clear elm_dm

   for i=1:m+1
     for j=1:m+1
       elm_dm(i,j)= 0;
     end
   end

   for i=2:m
    elm_dm(i,i) = 4*(i-1)^2 * i^2/((2*i-1)*h(l));
   end

   elm_dm(1,1)   =1/h(l); elm_dm(1,m+1)  =-1/h(l);
   elm_dm(m+1,1)=-1/h(l); elm_dm(m+1,m+1)= 1/h(l);

%---------------------------------------------->
% some optional computations
% demonstrating the connection between the modal
% and nodal expansion
%
% vd: Vandermonde matrix
%---

  clear vd

  for i=1:m+1
   for j=1:m+1
    mm(i,j) = elm_mm(i,j);
    dm(i,j) = elm_dm(i,j);
    vd(i,j) = 0.0;
   end
  end

%-----
% compute the vandermonde matrix
%------

  vd(1,1)=1; vd(m+1,m+1)=1;

%---
   if(m==2)
     vd(1,2)=1/2; vd(2,2)=1; vd(3,2)=1/2;
   elseif(m==3)
     xil(2)=-1/sqrt(5); xil(3)= 1/sqrt(5);
     for p=2:3
       xi = xil(p);
       vd(1,p)=(1-xi)/2;
       vd(2,p)=(1-xi^2);
       vd(3,p)=3*(1-xi^2)*xi;
       vd(4,p)=(1+xi)/2;
     end
   elseif(m==4)
     xil(2)=-sqrt(3/7); xil(3)=0; xil(4)=sqrt(3/7);
     for p=2:4
       xi = xil(p);
       vd(1,p)=(1-xi)/2;
       vd(2,p)=(1-xi^2);
       vd(3,p)=3*(1-xi^2)*xi;
       vd(4,p)=3*(1-xi^2)*(5*xi^2-1)/2;
       vd(5,p)=(1+xi)/2;
     end
  else
     disp(' -->');
     disp(' max polynomial order is 4');
     error(' sdsm_sys: Sorry this high order not yet implemented');
  end

%-----
% compute and print the nodal matrices
%------

%  dm_nodal = inv(vd)*dm*inv(vd')
%  mm_nodal = inv(vd)*mm*inv(vd')

%---------- end of ancillary computations
%---------------------------------------------->

% assemble:

   for i=1:m+1
      i1 = c(l,i);
      for j=1:m+1
       j1 = c(l,j);
       gdm(i1,j1) = gdm(i1,j1) + elm_dm(i,j);
       b(i1)      = b(i1)      + elm_mm(i,j)*s(j1)/k;
       gdm(i1,j1) = gdm(i1,j1) - helm*elm_mm(i,j); % optional for the Helm eq
      end
   end

%--
end   % end of loop over elements
%--

%----------
% last node
%----------

m = np(ne);

for i=1:m
  i1 = c(ne,i);
  b(i1) = b(i1) - elm_dm(i,m+1)*fL;
end

%-----
% done
%-----

return;
