%============================================
% CODE: sds
%
% Steady 1D diffusion with spectral elements
%===========================================

%-----------
% input data
%-----------

L=1.0; k=1.0; fL=0; q0=-1; 
helm=0; % Helmholtz constant: set to 0 for Poisson

%---
% case studies
%
fL=1/26.0; q0=-100/26^2;    % CASE R and R1
fL=0; q0=-pi; helm=pi^2;    % CASE H
fL=exp(1.0); q0=-1.0;       % CASE B
%---

ne=2; ratio=1.0;               % number of elements, stretch ratio

for i=1:ne
 np(i)=4;   % default element polynomial order
end

%-----
% trap
%-----

for i=1:ne
   if(np(i)>6)
   disp(' -->');
   disp(' max polynomial order is 6');
   error(' sds: Sorry this high order not yet implemented');
   end
end

%------------------------
% element node generation 
%------------------------

[xe,xen,xg,c,ng] = discr_s (0,L,ne,ratio,np);

%-------------------
% specify the source
%-------------------

for i=1:ng
  s(i) = 10.0*exp(-5.0*xg(i)^2/L^2);
  s(i) = -exp(xg(i));                    % CASE B
  s(i) = 200*(1-75*(2*xg(i)-1)^2)/(1+25*(2*xg(i)-1)^2)^3 ;   % case R1
  s(i) = 0;         % CASE H
  s(i) = 1.0/(1+25*(2*xg(i)-1)^2);       % CASE R
end

%-----------------
% element assembly
%-----------------

[gdm,b] = sds_sys (ne,xe,np,ng,c,q0,fL,k,s,helm);

%--------------
% linear solver
%--------------

gdm(:,ng) = [];  % remove the last (ng) column

gdm(ng,:) = [];  % remove the last (ng) row

b(:,ng) = [];    % remove the last (ng) element

f = b/gdm';       % solve the linear system

f = [f fL];      % add the value at the right end

%-----
% plot
%-----

plot(xg, f,'-+');hold on;
 ye= zeros(ne+1,1); plot(xe,ye,'o');
xlabel('x'); ylabel('f');

%-----
% done
%-----
