%==============================================
% CODE sdsc.m
%
% Steady 1D diffusion with spectral elements
% Condensed formulation
%==============================================

%-----------
% input data
%-----------

L=1.0; k=1.0; q0 = -1.0; fL=0.1;

ne=3; ratio=5.0;             % number of elements, stretch ratio
np(1)=5; np(2)=5; np(3)=3;   % element expansion order

%------------------------
% element node generation 
%------------------------

[xe,xen,xg,c,ng] = discr_s(0,L,ne,ratio,np);

%-------------------
% specify the source
%-------------------

for i=1:ng
 s(i) = 10.0*exp(-5.0*xg(i)^2/L^2);
end

%-----------------
% element assembly
%-----------------

[gdm,b] = sdsc_sys (ne,xe,np,c,q0,fL,k,s);

%--------------
% linear solver
%--------------

gdm(:,ne+1) = [];  % remove the last (ne+1) column
gdm(ne+1,:) = [];  % remove the last (ne+1) row
b(:,ne+1) = [];    % remove the last (ne+1) node

f = b/gdm';       % solve the linear system

f = [f fL];      % add the value at the right end

%-----
% plot
%-----

plot(xe, f,'x'); hold on;
plot(xe, f);
xlabel('x');ylabel('f');
ye= zeros(ne+1,1); plot(xe,ye,'o');

%-----
% done
%-----
