function [Z, W] = lobatto(i)

%=================================================
% Zeros of the ith-degree Lobatto polynomial,
% and corresponding weights for the Lobatto
% integration quadrature
%
% This table contains values for i = 1, 2, ..., 6
% The default value is i=6
%=================================================

%------
% check
%------

if(i>6)

  disp(' -->');
  disp(' lobatto: Chosen number of points');
  disp('   is not available; Will take i=6');
  i=6;

end

%-------
if(i==1)
%-------

Z(1) = 0.0; W(1) = 4.0/3.0;

%-----------
elseif(i==2)
%-----------

Z(1) = -1.0/sqrt(5.0); Z(2) = -Z(1);
W(1) = 5.0/6.0; W(2) = W(1);

%-----------
elseif(i==3)
%-----------

Z(1) = -sqrt(3.0/7.0); Z(2) = 0.0; Z(3) = -Z(1);
W(1) = 49.0/90.0; W(2) = 32.0/45.0; W(3) = W(1);

%-----------
elseif(i==4)
%-----------

Z(1) = -0.76505532392946; Z(2) = -0.28523151648064;
Z(3) = -Z(2); Z(4) = -Z(1);

W(1) = 0.37847496; W(2) = 0.55485838;
W(3) = W(2); W(4) = W(1);
      
%-----------
elseif(i==5)
%-----------

Z(1) = -0.83022389627857; Z(2) = -0.46884879347071;
Z(3) = 0.0; Z(4) = -Z(2); Z(5) = -Z(1);
W(1) = 0.27682604; W(2) = 0.43174538;
W(3) = 0.48761904; W(4) = W(2); W(5) = W(1);

%-----------
elseif(i==6)
%-----------

Z(1) = - 0.87174015; Z(2) = - 0.59170018; 
Z(3) = - 0.20929922;
Z(4) = -Z(3); Z(5) = -Z(2); Z(6) = -Z(1);
W(1) = 0.21070422; W(2) = 0.34112270;
W(3) = 0.41245880; W(4) = W(3);
W(5) = W(2); W(6) = W(1);

%--
end
%--

return;
