%================================================
% Lobatto node distribution in the tetrahedron
% corresponding to a complete mth order expansion
%================================================

m = 5

%-----
% draw the edges
%-----

x(1) = 0.0; y(1)= 0.0; z(1)= 0.0;
x(2) = 1.0; y(2)= 0.0; z(2)= 0.0;
x(3) = 0.0; y(3)= 1.0; z(3)= 0.0;
x(4) = 0.0; y(4)= 0.0; z(4)= 1.0;
                                                                                
% edge node labels

s(1,1)=2; s(1,2)=3; s(1,3)=4;
s(2,1)=3; s(2,2)=1; s(2,3)=4;
s(3,1)=1; s(3,2)=2; s(3,3)=4;
s(4,1)=1; s(4,2)=2; s(4,3)=3;

% draw the edges

for i=1:4
 for j=1:3
  xpp(j)=x(s(i,j)); ypp(j)=y(s(i,j)); zpp(j)=z(s(i,j));

  % regular terahedron:
   xpp(j)=xpp(j)+0.5*(ypp(j)+zpp(j));
   ypp(j)=0.5*sqrt(3)*(ypp(j)+zpp(j)/3.0);
   zpp(j) = sqrt(2/3)*zpp(j);

 end
 hold on;
 plot3(xpp,ypp,zpp,'linewidth',2)
end

xlabel('x'); ylabel('y'); zlabel('z');



%-------------
% lobatto grid
%-------------
                                                                                
[t, W] = lobatto(m-1);    % W will not be needed
                                                                                
v(1) = 0.0;
for i=2:m
 v(i) = 0.5*(1.0+t(i-1));
end
v(m+1) = 1.0;

%--------------------
% uniform master grid
%--------------------

%for i=1:m+1
%  v(i) = (i-1.0)/m;
%end

%---------------------------
% deploy and count the nodes
%---------------------------

Ic = 0; % node counter

hold on

%---
% xi-eta face
%---

for i=1:m+1;
 for j=1:m+2-i;
    l = m+3-i-j;
    xp = (1.0+2.0*v(i)-    v(j)-v(l))/3.0;
    yp = (1.0-    v(i)+2.0*v(j)-v(l))/3.0;
    zp = 0.0;
    Ic = Ic+1;
    % regular tetrahedron:
    xp=xp+0.5*(yp+zp); yp=0.5*sqrt(3)*(yp+zp/3.0);
    zp = sqrt(2/3)*zp;
    plot3(xp,yp,zp,'+');
 end
end

pause(1)

%---
% eta-zeta face
%---

for j=1:m;
 for k=2:m+2-j;
    l = m+3-j-k;
    xp = 0.0;
    yp = (1.0+2.0*v(j)-    v(k)-v(l))/3.0;
    zp = (1.0-    v(j)+2.0*v(k)-v(l))/3.0;
    Ic = Ic+1;
    % regular tetrahedron:
    xp=xp+0.5*(yp+zp); yp=0.5*sqrt(3)*(yp+zp/3.0);
    zp = sqrt(2/3)*zp;
    plot3(xp,yp,zp,'o');
 end
end

pause(1)

%---
% zeta-xi face
%---

for i=2:m;
 for k=2:m+2-i;
    l = m+3-i-k;
    xp = (1.0+2.0*v(i)-    v(k)-v(l))/3.0;
    yp = 0.0;
    zp = (1.0-    v(i)+2.0*v(k)-v(l))/3.0;
    Ic = Ic+1;
    % regular tetrahedron:
    xp=xp+0.5*(yp+zp); yp=0.5*sqrt(3)*(yp+zp/3.0);
    zp = sqrt(2/3)*zp;
    plot3(xp,yp,zp,'x');
 end
end

pause(1)

%---
% slanted face
%---

for i=2:m;
 for j=2:m+1-i;
    l = m+3-i-j;
    xp = (1.0+2.0*v(i)-     v(j)-v(l))/3.0;
    yp = (1.0-    v(i)+ 2.0*v(j)-v(l))/3.0;
    zp = 1.0-xp-yp;
    Ic = Ic+1;
    % regular tetrahedron:
    xp=xp+0.5*(yp+zp); yp=0.5*sqrt(3)*(yp+zp/3.0);
    zp = sqrt(2/3)*zp;
    plot3(xp,yp,zp,'p');
 end
end

pause(1)

%---

count = 0;

for i=2:m;
 for j=2:m+1-i
  for k=2:m+2-i-j

    l = m+4-i-j-k;
    xp = (1.0+3.0*v(i)-    v(j)-    v(k)-v(l))/4.0;
    yp = (1.0-    v(i)+3.0*v(j)-    v(k)-v(l))/4.0;
    zp = (1.0-    v(i)-    v(j)+3.0*v(k)-v(l))/4.0;
    Ic = Ic+1;
    % regular tetrahedron:
    xp=xp+0.5*(yp+zp); yp=0.5*sqrt(3)*(yp+zp/3.0);
    zp = sqrt(2/3)*zp;
    plot3(xp,yp,zp,'h','markersize',10,'color','red');
  end
 end
end

%---
% number of nodes
%---
Ic
formula = (m+1)*(m+2)*(m+3)/6.0
