%========================
% tetra4
%
% visualize a tetrahedron
%========================

% arbitrary vertex coordinates:

  x(1) = 0.0; y(1)= 0.0; z(1)= 0.0;
  x(2) = 0.9; y(2)= 0.2; z(2)= 0.5;
  x(3) = 0.1; y(3)= 1.1; z(3)=-0.05;
  x(4) =-0.1; y(4)=-0.1; z(4)= 1.2;

% canonical vertex coordinates:

 x(1) = 0.0; y(1) = 0.0; z(1)= 0.0;
 x(2) = 1.0; y(2) = 0.0; z(2)= 0.0;
 x(3) = 0.0; y(3) = 1.0; z(3)= 0.0;
 x(4) = 0.0; y(4) = 0.0; z(4)= 1.0;

% face node labels:

s(1,1)=3; s(1,2)=2; s(1,3)=4;    % face 1
s(2,1)=1; s(2,2)=3; s(2,3)=4;    % face 2
s(3,1)=1; s(3,2)=4; s(3,3)=2;    % face 3
s(4,1)=1; s(4,2)=2; s(4,3)=3;    % face 4

% paint the faces:

option=1;

for i=1:4

 for j=1:3

  xv = x(s(i,j));
  yv = y(s(i,j));
  zv = z(s(i,j));

  xp(j)=xv; yp(j)=yv; zp(j)=zv;

%  canonical regular tetrahedron:
%  xp(j)=xv+0.5*(yv+zv); yp(j)=0.5*sqrt(3)*(yv+zv/3.0); 
%  zp(j) = sqrt(2/3)*zv;

 end
    if(option==1) % solid
       if(i==1) patch(xp,yp,zp,'r'); end
       if(i==2) patch(xp,yp,zp,'b'); end
       if(i==3) patch(xp,yp,zp,'y'); end
       if(i==4) patch(xp,yp,zp,'g'); end
     else    % wireframe
       plot3 (xp,yp,zp); hold on;
     end
end

xlabel('x'); ylabel('y'); zlabel('z');
axis([-0.3 1.3 -0.3 1.3 -0.3 1.3]);

%----------------
% Jacobian matrix
%----------------

Jac(1,1)=x(2)-x(1); Jac(1,2)=x(3)-x(1); Jac(1,3)=x(4)-x(1);
Jac(2,1)=y(2)-y(1); Jac(2,2)=y(3)-y(1); Jac(2,3)=y(4)-y(1);
Jac(3,1)=z(2)-z(1); Jac(3,2)=z(3)-z(1); Jac(3,3)=z(4)-z(1);

%-------
% Volume
%-------

V = det(Jac)/6.0;

%----------------------------------------------
% gradients of the node interpolation functions
%----------------------------------------------

hold on;

V6 = 6.0*V;

x21 = x(2)-x(1); y21 = y(2)-y(1); z21 = z(2)-z(1);
x31 = x(3)-x(1); y31 = y(3)-y(1); z31 = z(3)-z(1);
x41 = x(4)-x(1); y41 = y(4)-y(1); z41 = z(4)-z(1);
x32 = x(3)-x(2); y32 = y(3)-y(2); z32 = z(3)-z(2);
x42 = x(4)-x(2); y42 = y(4)-y(2); z42 = z(4)-z(2);

% first gradient 

px(1) = (x(2)+x(3)+x(4))/3.0;
py(1) = (y(2)+y(3)+y(4))/3.0;
pz(1) = (z(2)+z(3)+z(4))/3.0;
px(2) = (-y32*z42 + z32*y42)/V6;
py(2) = ( x32*z42 - z32*x42)/V6;
pz(2) = (-x32*y42 + y32*x42)/V6;
mag = sqrt(px(2)^2+py(2)^2+pz(2)^2); fc = -0.10/mag;
%  quiver3(px(1),py(1),pz(1),fc*px(2),fc*py(2),fc*pz(2),'ob')
px(2)=px(1)+fc*px(2);
py(2)=py(1)+fc*py(2);
pz(2)=pz(1)+fc*pz(2);
plot3(px,py,pz,'-ob')

% second gradient 

px(1) = (x(1)+x(3)+x(4))/3.0;
py(1) = (y(1)+y(3)+y(4))/3.0;
pz(1) = (z(1)+z(3)+z(4))/3.0;
px(2) = ( y31*z41 - z31*y41)/V6;
py(2) = (-x31*z41 + z31*x41)/V6;
pz(2) = ( x31*y41 - y31*x41)/V6;
mag = sqrt(px(2)^2+py(2)^2+pz(2)^2); fc = -0.10/mag;
%quiver3(px(1),py(1),pz(1),fc*px(2),fc*py(2),fc*pz(2),'or')
px(2)=px(1)+fc*px(2);
py(2)=py(1)+fc*py(2);
pz(2)=pz(1)+fc*pz(2);
plot3(px,py,pz,'-or')

% third gradient 

px(1) = (x(1)+x(2)+x(4))/3.0;
py(1) = (y(1)+y(2)+y(4))/3.0;
pz(1) = (z(1)+z(2)+z(4))/3.0;
px(2) = (-y21*z41 + z21*y41)/V6;
py(2) = ( x21*z41 - z21*x41)/V6;
pz(2) = (-x21*y41 + y21*x41)/V6;
mag = sqrt(px(2)^2+py(2)^2+pz(2)^2); fc = -0.10/mag;
%quiver3(px(1),py(1),pz(1),fc*px(2),fc*py(2),fc*pz(2),'og')
px(2)=px(1)+fc*px(2);
py(2)=py(1)+fc*py(2);
pz(2)=pz(1)+fc*pz(2);
plot3(px,py,pz,'-og')

% fourth gradient 

px(1) = (x(1)+x(2)+x(3))/3.0;
py(1) = (y(1)+y(2)+y(3))/3.0;
pz(1) = (z(1)+z(2)+z(3))/3.0;
px(2) = ( y21*z31 - z21*y31)/V6;
py(2) = (-x21*z31 + z21*x31)/V6;
pz(2) = ( x21*y31 - y21*x31)/V6;
mag = sqrt(px(2)^2+py(2)^2+pz(2)^2); fc = -0.10/mag;
%quiver3(px(1),py(1),pz(1),fc*px(2),fc*py(2),fc*pz(2),'ok')
px(2)=px(1)+fc*px(2);
py(2)=py(1)+fc*py(2);
pz(2)=pz(1)+fc*pz(2);
plot3(px,py,pz,'-ok')

%-----
% done
%-----
